
From: Russell King <rmk@arm.linux.org.uk>

There appears to be one case missing from the wait_event() family - the
uninterruptible timeout wait.  The following patch adds this.

This wait is particularly useful when (eg) you wish to pass work off to an
interrupt handler to perform, but also want to know if the hardware has
decided to go gaga under you.  You don't want to sit around waiting for
something that'll never happen - you want to go and wack the gremlin which
caused the failure and retry.

Signed-off-by: Andrew Morton <akpm@osdl.org>
---

 25-akpm/include/linux/wait.h |   23 +++++++++++++++++++++++
 1 files changed, 23 insertions(+)

diff -puN include/linux/wait.h~add-wait_event_timeout include/linux/wait.h
--- 25/include/linux/wait.h~add-wait_event_timeout	2004-09-25 22:04:22.425329248 -0700
+++ 25-akpm/include/linux/wait.h	2004-09-25 22:04:22.429328640 -0700
@@ -176,6 +176,29 @@ do {									\
 	__wait_event(wq, condition);					\
 } while (0)
 
+#define __wait_event_timeout(wq, condition, ret)			\
+do {									\
+	DEFINE_WAIT(__wait);						\
+									\
+	for (;;) {							\
+		prepare_to_wait(&wq, &__wait, TASK_UNINTERRUPTIBLE);	\
+		if (condition)						\
+			break;						\
+		ret = schedule_timeout(ret);				\
+		if (!ret)						\
+			break;						\
+	}								\
+	finish_wait(&wq, &__wait);					\
+} while (0)
+
+#define wait_event_timeout(wq, condition, timeout)			\
+({									\
+	long __ret = timeout;						\
+	if (!(condition)) 						\
+		__wait_event_timeout(wq, condition, __ret);		\
+	__ret;								\
+})
+
 #define __wait_event_interruptible(wq, condition, ret)			\
 do {									\
 	DEFINE_WAIT(__wait);						\
_
